#pragma once
#include "exportrules.h"
#include "codeword.h"

#include <string>
#include <unordered_map>
#include <vector>

class SSTD_API Codebook {
	std::unordered_map<std::string, unsigned int> str2label;
	std::vector<std::string> label2str;
	
	std::vector<Codeword> codewords;

	unsigned int getLabel(const std::string & label);

public:

	Codebook() = default;
	Codebook(const Codebook & rhs) = default;
	Codebook(Codebook && rhs) = default;
	
	Codebook & operator=(const Codebook &) = default;
	Codebook & operator=(Codebook &&) = default;
	
	virtual ~Codebook() = default;
	
	void addCodeword( const Vector & vec, const std::string & label);
	void addCodeword( Vector && vec, const std::string & label);
	
	void combine( const Codebook & other ); 

	const std::string & getLabelString(const Codeword &cword) const;
	const std::string & getLabelString(unsigned int labelId) const;

	unsigned int getLabelId(const std::string & label) const  {
		auto it = str2label.find(label);
		return it == str2label.end() ? Codeword::INVALID_LABEL : it->second;
	}

	Codeword & operator[]( unsigned int index) {
		return codewords[index];
	}

	const Codeword & operator[]( unsigned int index) const {
		return codewords[index];
	}

	unsigned int size() const {
		return codewords.size();
	}

	// Implementing iterator template interface
	using iterator = std::vector<Codeword>::iterator;
	using const_iterator = std::vector<Codeword>::const_iterator;

	iterator begin() 	{ return codewords.begin(); }
	iterator end() 		{ return codewords.end();   }

	const_iterator begin() 	const { return codewords.begin(); }
	const_iterator end() 	const { return codewords.end();   }

	const_iterator cbegin() const { return begin(); }
	const_iterator cend() 	const { return end();   }

    /** Depreciated, use getLabelId and getLabelString instead */
	auto getLabelMap() const -> const decltype(str2label) & {return str2label;}

	friend SSTD_API std::ostream & operator<<( std::ostream & out, const Codebook & rhs);
};



SSTD_API void 	  saveCodebook( const std::string & fname, const Codebook & cb );
SSTD_API Codebook  loadCodebook( const std::string & fname );


