#pragma once
#include "vector.h"

class SSTD_API Codeword : public Vector {
	private:
		uint32_t _label;
	public:
		enum : uint32_t { INVALID_LABEL = 0xFFFFFFFF };
		Codeword() : Vector() , _label(INVALID_LABEL) {}
		Codeword(const Vector & vec, uint32_t label) : Vector(vec), _label(label) {}
		Codeword(Vector && vec, uint32_t label) : Vector(std::forward<Vector&&>(vec)), _label(label) {}
		
		Codeword(const Codeword &) = default;
		Codeword(Codeword &&) = default;
		Codeword & operator=(const Codeword &) = default;
		Codeword & operator=(Codeword &&) = default;
		
		Codeword & operator=(const Vector & rhs) {
		    Vector::operator=(rhs);
		    return *this;
		}
		
		Codeword & operator=(Vector && rhs) {
		    Vector::operator=(std::forward<Vector&&>(rhs));
		    return *this;
		}
		
		uint32_t label() const {
			return _label;
		}
		
		void label(uint32_t label){
			_label = label;
		}
		
		/** If readFrom or write to returns false. The Codeword may be in an invalid state. */
		bool readFrom(std::istream & in);
		bool writeTo(std::ostream & out) const;
		

};

SSTD_API std::ostream & operator<<(std::ostream & out, const Codeword & obj );
