# The tests in this file sets the HOST, TARGET, OUTDIR and CROSSCOMPILE variables
CC 	:= gcc
CXX := g++
AS  := as

ifndef HOST
	ifeq ($(shell uname -m),x86_64)
	HOST := x86
	else 
		ifeq ($(shell uname -m),armv7l)
			HOST := arm
		else 
			@echo "Could not determine host."
			return
		endif
	endif
endif

ifeq ($(HOST),arm)
	TARGET := arm
	OUTDIR := $(HOME)/bin/
	LIBDIR := $(abspath $(OUTDIR))
endif

ifeq ($(HOST),x86)
	ifeq ($(TARGET),arm)
		CROSS_COMPILE := arm-linux-gnueabihf-
		OUTDIR := ../bin_arm/
		LIBDIR := /home/stian/bin/
	else
		TARGET  := x86
		OUTDIR := $(HOME)/bin/
		LIBDIR := $(abspath $(OUTDIR))
	endif
endif

ifeq ($(TARGET),arm)
	CFLAGS   := -march=armv7-a -mcpu=cortex-a9 -marm -mfpu=neon -mtune=cortex-a9 -mfloat-abi=hard
endif


# DEBUG options
ifeq ($(DEBUG),1)
	CFLAGS += -gdwarf-3 -Og
else
	CFLAGS += -O3
endif
