#This script genereates the assembly for the spy-process given different cache parameters.




fileheader = """/* AUTOMATICALLY GENERATED */
#include "asm_defines.h"
#include "types.h"

.text
.globl spy_loop_abs

/* void spy_loop_abs(  abs_t * raw_timings, unsigned long N) */

"""





def main():

	cache_size =        int(input_or_default('Enter Cache Size in kB', 32)) * 1024;
	cache_line_size =   int(input_or_default('Enter Cache Line Size in bytes', 32));
	cache_assoc	=       int(input_or_default('Enter Cache Associativity', 4));
	
	cache_sets = cache_size // (cache_line_size*cache_assoc);

	print ('Generating For the following cache:');
	print ('Cache Size:\t', cache_size);
	print ('Cache Sets:\t', cache_sets);
	print ('Cache Assoc:\t', cache_assoc);
	print ('Line Size:\t', cache_line_size);
	
	ans = input_or_default('Is this ok?', 'yes');

	if (ans == 'yes'):
		with open('asm_defines.h','w') as f:
			#Defines:
			f.write("""#pragma once
/****************************************
 * This is an autogenerated file, any   *
 * changes you make will be overwritten *
 ***************************************/
""");

			f.write("#define CACHE_SETS\t"+str(cache_sets) + "\n");
			f.write("#define CACHE_LINE\t"+str(cache_line_size) + "\n");
			f.write("#define CACHE_ASSOC\t"+str(cache_assoc) + "\n");
			f.write("#define CACHE_SIZE\t"+str(cache_size) + "\n\n");
			
			print('Generated file: asm_defines.h');

		with open('spy_loop.S','w') as f:
			f.write("/*This file was generated with parameters: Cache size: "+str(cache_size) + "bytes, Cache line size: " + str(cache_line_size) + " bytes and cache associativity: "+ str(cache_assoc) + "*/\n\n")
			
			f.write(fileheader);
			
			# SPY_ABS
			f.write("spy_loop_abs:\n")
			f.write("""
	/* R0 holds argument raw_timings and will hold N*N_CACHE_SETS values. */
	/* R1 holds argument N which is number of rounds*/
	/* R2 holds current measurement */
	/* R3 holds previous measuremnt */

	/* PROLOG */
	/* Start */
	MOV R2, #CACHE_SETS
	MUL	R1, R1, R2 
 
	MOV R2, #RAW_T_SIZE
	MLA R1, R1, R2, R0  /* R1 = R1*R2 + R0 */
	
	/* Save first timing. */
	MRC p15, 0, R3, c9, c13, 0
""");
			f.write(".balign " + str(cache_size//cache_assoc) + '\n');

			label = 0;
			for ass in range(cache_assoc-1):
				for cset in range(cache_sets):
					f.write('L' + format(label, "05d")+':\n');
					f.write('\tB L' + format(label + cache_sets, "05d") + '\n');
					f.write('\t.rept '+str(cache_line_size//4-1)+'\n\t\tNOP\n\t.endr\n');
					label += 1;
			
			for cset in range(cache_sets-1):
				f.write('L' + format(label, "05d")+':\n');
				f.write('\tMRC p15, 0, R2, c9, c13, 0\n');
				f.write('\tCMP R2, R3\n');
				f.write('\tSUBLE R3, R3, R2\n');
				f.write('\tSUBGT R3, R2, R3\n');
				f.write('\tSTR R3, [R0], #4\n')
				f.write('\tMRC p15, 0, R3, c9, c13, 0\n');
				f.write('\tB L' + format((label % cache_sets + 1), "05d") + '\n');
				f.write('\tNOP\n');
				label += 1

			
			f.write('L' + format(label, "05d")+':\n');
			f.write('\tMRC p15, 0, R2, c9, c13, 0\n');
			f.write('\tCMP R2, R3\n');
			f.write('\tSUBLE R3, R3, R2\n');
			f.write('\tSUBGT R3, R2, R3\n');
			f.write('\tSTR R3, [R0], #4\n')
			f.write('\tMRC p15, 0, R3, c9, c13, 0\n');
			f.write('\tCMP R0, R1   /* This is ok, since we are only comparing for whole cache sets.*/\n')		
			f.write('\tBLO L' + format(0, "05d")+"\n");			
			f.write('\tB END\n')
			

			f.write("""END:

	/* EPILOG */

	MOV pc, lr
""");

			print('Generated file: spy_loop.S');

		exit(0);

	else:
		print('Aborting');
		exit(1);


def input_or_default( prompt, default ):
	val = input(prompt + ' [' + str(default) + ']: ');
	if (val == ''):
		return default;
	else:
		return val;

main();
