/* 	Format
  	Magic Constant: "CODEBOOK"
	STX   	
	LABELS separated by NULL
	ETX
   	CODEWORDS tightly packed
 */

#include "codebook.h"
#include <stdexcept>
#include <string>
#include <cstring>
#include <fstream>
#include <map>

using namespace std;

// Special characters
const char NUL = 0x00; // '\0'
const char STX = 0x02; // Start text
const char ETX = 0x03; // End text
const char US  = 0x1f; // Unit separator

static const char * MAGIC_STRING = "CODEBOOK";


void SSTD_API saveCodebook( const std::string & fname, const Codebook & cb ) {
	ofstream fout(fname, ios::binary | ios::trunc );
	if (!fout) {
		throw runtime_error("Could not open file " + fname );
	}

	unsigned int temp;

	fout.write(MAGIC_STRING, 8);
	fout.write(&STX, 1);

	std::map<unsigned int, std::string> sorted_labels;
	for (auto &elem : cb.getLabelMap() ) {
		sorted_labels[elem.second] = elem.first;
	}

	for (auto &elem : sorted_labels ) {
		fout.write(elem.second.c_str(), elem.second.size()+1); // Include '\0'
	}
	fout.write(&ETX, 1);

	for (auto &cw : cb ) {
		cw.writeTo(fout);
	}
}
	
Codebook SSTD_API loadCodebook( const std::string & fname ) {
	ifstream fin(fname, ios::in | ios::binary );
	if (!fin) {
		throw runtime_error("Could not open file " + fname );
	}

	vector<string> labels;

	{
		const int BUF_SZ = 9;
		char buffer[BUF_SZ];
		buffer[BUF_SZ-1] = '\0';
	
	
		fin.read(buffer, BUF_SZ-1);
		if( strcmp(buffer, MAGIC_STRING) ) {
			throw logic_error("File " + fname + " is not a codebook file.");
		}
	}

	{   // Read all labels
		string templabel;
		fin.get(); // ignore STX
		char ch = fin.get(); 
		while ( ch != ETX ) {
			while ( ch != NUL ) {
				templabel.push_back(ch);
				ch = fin.get();			
			}
			labels.push_back(templabel);
			templabel.clear();
			ch = fin.get();
		}
	}

	Codeword cw;
	Codebook cb;
	while (cw.readFrom(fin)) {
		cb.addCodeword( cw, labels[cw.label()]);
	}

	return cb;
}
