#include "codebook.h"
#include <ostream>
#include <utility>

unsigned int Codebook::getLabel(const std::string & label) {
	auto it = str2label.find(label);
	if ( it == str2label.end() ) {
		label2str.push_back(label);
	    it = str2label.emplace( label, str2label.size()).first;
	}
	return it->second;
}

void Codebook::addCodeword( const Vector & vec, const std::string & label) {
	unsigned int labelid = getLabel(label);
	codewords.emplace_back(vec, labelid);
}

void Codebook::addCodeword( Vector && vec, const std::string & label) {
	unsigned int labelid = getLabel(label);
	codewords.emplace_back(std::forward<Vector&&>(vec), labelid);
}

const std::string & Codebook::getLabelString(const Codeword &cword) const  {
	return getLabelString(cword.label());
}

const std::string & Codebook::getLabelString(unsigned int labelId) const  {
	return label2str.at(labelId);
}

void Codebook::combine( const Codebook & other ) {
	// Load codewords.
	for (auto &cw : other ) {
	    this->addCodeword( cw, other.getLabelString(cw) );
	}
}

std::ostream & operator<<( std::ostream & out, const Codebook & rhs) {
	for ( auto cw : rhs.codewords ) {
		out << cw << '\n'; 
	}
	return out;
}



