#include "codebook_reduce.h"
#include "SOM.h"

#include <algorithm>
#include <set>
#include <iostream> //REMOVE


using namespace std;

template <class T>
struct SSTD_LOCAL VectorComparator {
   bool operator()(const VectorT<T> & lhs, const VectorT<T> & rhs ) {
       assert(lhs.size() == rhs.size());

       for (int i = 0; i < lhs.size(); i++ ) {
           if (lhs[i] >= rhs[i]) {
               return false;
           }
       }
       return true;
   }
};

SSTD_API std::vector<unsigned int> find_significant_components( const std::vector<const Codebook*> & codebooks ) {
	vector<dVec> mean;
	dVec smallest_var;
	int baseline = 0;
	
	if (codebooks.size() == 0)
		return vector<unsigned int>();

	int ncomp = (*(codebooks[0]))[0].size();

	// Calculate mean and variance for all codebooks.
	int i = 0;
	for (const Codebook * cb : codebooks ) {
		
	    dVec centroid = dVec(ncomp, 0.0);
	    for ( auto & cw : *cb ) {
			assert (cw.size() == ncomp && "Components must match across all elements.");
	    	centroid += cw.convertTo<double>();
	    } 
	    centroid *= 1.0/cb->size();
	    mean.push_back(centroid);

		dVec variance = dVec(ncomp, 0.0);
	    for ( auto & cw : *cb ) {
	    	dVec temp = centroid - cw.convertTo<double>();
			for (int i = 0; i < temp.size(); i++) {
				temp[i] *= temp[i];
			}
			variance += temp;
	    } 

		variance *= 1.0/(cb->size()-1); // Unbiased estimate of population variance
		variance *= 1.0/(cb->size());   // Sample variance
		
		// Choose the codebook with the smallest variance as baseline
		if (smallest_var.size() == 0 || variance.length() < smallest_var.length()) {
			smallest_var = variance;
			baseline = i;
		}

		++i;
	}

	// Include any cache set significanly different from the baseline 99% confidence.
	const double TWO_TAILED_99 = 2.576;
	vector<unsigned int> sig_comp;
	for ( i = 0; i < ncomp; i++) {
		for (int icb = 0; icb < mean.size(); icb++) {
			if ( icb == baseline)
				continue;
			
			double Zval = fabs(mean[icb][i] - mean[baseline][i])/sqrt(smallest_var[i]);
			if (Zval > TWO_TAILED_99){
				sig_comp.push_back(static_cast<unsigned int >(i));
				break;
			}
		}
	}
	
	return std::move(sig_comp);
}

SSTD_API Vector reduce_dimensions(const Vector & vec, const std::vector<unsigned int> & components ) {
    Vector ret(components.size());
    unsigned int i = 0;
    for ( unsigned int c : components ) {
       ret[i] = vec[c];
       i++;
    }
    return std::move(ret);    
}

