#include "lvq.h"
#include "progress_callback.h"

#include <vector>
#include <utility>


using namespace std;

bool LVQ::isInWindow( const DistanceInfo & m1, const DistanceInfo & m2 , double windowsize) {
	double dd1 = (double)m1.distance/m2.distance;
	double dd2 = (double)m2.distance/m1.distance;
	return min(dd1, dd2) > windowsize;
}
	
	
void LVQ::lvq2_correct( Codeword * sameClass, Codeword * diffClass, const Codeword & word, double alpha ) {
	// check that word lies between conflicting codewords.
	if ( sameClass->label() == word.label() && diffClass->label() != word.label() ) {
		*sameClass 	+= alpha*(word - *sameClass);
		*diffClass 	-= alpha*(word - *diffClass);
	}
}

void LVQ::train_lvq1( const Codeword & word, double alpha ) {
	DistanceInfo candidate = getClosestCodeword( word );
	
	if (candidate.cword->label() == word.label()) // Correct classification
		*candidate.cword += alpha * (word - *candidate.cword);
	else // Incorrect clasification
		*candidate.cword -= alpha * (word - *candidate.cword);		
}
	
	
void LVQ::train_lvq2( const Codeword & word, double alpha, double window ) {
	auto candidates = getClosest2Codewords(word);

	if (isInWindow( candidates.first, candidates.second, window )) {		
		lvq2_correct( candidates.first.cword, candidates.second.cword, word, alpha);
	}
}

void LVQ::train_lvq2_1( const Codeword & word, double alpha, double window ) {
	auto candidates = getClosest2Codewords(word);

	if (isInWindow( candidates.first, candidates.second, window )) {
		if ( candidates.first.cword->label() != word.label() ) {
			swap(candidates.first, candidates.second); 
		}
	
		lvq2_correct( candidates.first.cword, candidates.second.cword, word, alpha);
	}
}

void LVQ::train_lvq3( const Codeword & word, double alpha, double window, double epsilon ) {
	auto candidates = getClosest2Codewords(word);

	if (isInWindow( candidates.first, candidates.second, window )) {		
		if ( candidates.first.cword->label() != word.label() ) {
			
			swap(candidates.first, candidates.second); 
		}
	
		if ( candidates.first.cword->label() == word.label() ) {
			if ( candidates.second.cword->label() != word.label() ) {
				lvq2_correct( candidates.first.cword, candidates.second.cword, word, alpha);
			}
			else {
				*candidates.first.cword 	+= epsilon*alpha*(word - *candidates.first.cword);
				*candidates.second.cword	+= epsilon*alpha*(word - *candidates.second.cword);
			}
		}
	}
}
	
	
	
void LVQ::train_vq( const Codebook & cb_in,unsigned int iterations, double window, double alpha, double epsilon ) {
	double alpha_step = alpha/(iterations);	
	
	if (iterations > 10000) {
	    cout << "Warning: LVQ should be used for a short period of time, Kohonen recommends at most 10 000 iterations." << endl;
	}

	// Translation table between the local codebook and the argument.
	std::vector<unsigned int> labels(cb_in.getLabelMap().size());

	for (auto &elem : cb_in.getLabelMap() ) {
		labels[elem.second] = getLabelId(elem.first);
	}

	for (int count = 0; count < iterations; count++ ) {
	    Codeword cw = cb_in[rand() % cb_in.size() ];
	    
		if (p_cb) { p_cb->updateProgress(static_cast<double>(count++*100)/iterations, "Training: ");}
		cw.label( labels[ cw.label()] );

		train_lvq3(cw, alpha, window , epsilon );
		alpha -= alpha_step;
	
	}
	if (p_cb) { p_cb->updateProgress(100, "Training: ");}
}
