#include "pgm.h"
#include <cstdint>
#include <fstream>
#include <sstream>
#include <iostream>
#include <string>
#include <stdexcept>
#include <cstring>
#include <cstdio>

#define _stringify(x) # x
#define stringify(x) _stringify(x)

std::string SSTD_LOCAL fix_filename(const char * filename, const char * filesuffix) {
	std::string temp(filename);
	if ( temp.size() < 4 || temp.substr(temp.size()-4) != filesuffix ) 
		temp += filesuffix;
	return temp;
}

using namespace std;

void SSTD_API pgm_save(const char * filename, const void * _data, unsigned int bitdepth, unsigned int width, unsigned int height) {
	using namespace std;

	const char * data = reinterpret_cast<const char *>(_data);

	unsigned int bytedepth = bitdepth / 8;

	
	const string fn = fix_filename(filename, ".pgm");
	FILE* fout = fopen(fn.c_str(), "wb");
	if (!fout) {
		throw runtime_error(std::string("Could not open file!") + fn);
	}

	/* Saving in PGM file format */
	ostringstream sout;
	sout << "P5\n";
	if (bytedepth <= 2) { 
			
   		sout << width << " " << height << " " << (1<<bitdepth)-1 << endl;
		if (!fwrite(sout.str().c_str(), 1, sout.str().size(), fout)) throw runtime_error(std::string("Error!"));

		size_t written = 0;
		while (written < width*height) {
			size_t temp = fwrite(data + written, 1, width*height*bytedepth - written, fout);
			if (!temp) throw runtime_error(string("Write failed."));
			written += temp;
		}
	}
	fclose(fout);
}


const char LF = 0x0a;
const char CR = 0x0d;

void SSTD_API pgm_load(const char * filename, void * data, unsigned int &bitDepth, unsigned int &width, unsigned int &height ) {
	using namespace std;

	const string fn = fix_filename(filename, ".pgm");
	ifstream fin(fn, ios::in | ios::binary );
	if (!fin) {
		throw runtime_error(std::string("Could not open file: ") + fn);
	}

	/* Loading in RAW file format */
	char buf[3] = {};
	fin.read(buf, 2);
	if( strcmp(buf, "P5")) {
		throw runtime_error(std::string("Not PGM file: ") + fn);
	}

	uint32_t w,h,max, d;
	
	fin >> w >> h >> max;
	if (max > 255) 
		d = 2;
	else
		d = 1;
	
	
	char ws = fin.get();				// ignore following whitespace
	if (ws == CR && fin.peek() == LF) { // Windows line endings.
		fin.get();
	}
		 
	
	if (data) {
		fin.read( reinterpret_cast<char*>(data), width*height*(d));
	}
	width = w;
	height = h;
	bitDepth = d*8;

	fin.close();
}
