#include "raw.h"
#include <iostream>
#include <fstream>
#include <string>
#include <cstdint>
#include <cstring>
#include <stdexcept>

using namespace std;

extern std::string SSTD_LOCAL fix_filename(const char * filename, const char * filesuffix);

void SSTD_LOCAL writeUInt32(ofstream & out, uint32_t val ) {
	unsigned char buff[4];
	for (int i = 0; i < 4; i++) {
		buff[i] = static_cast<unsigned char>( val & 0xff );
		val /= 0x100;
	}
	out.write(reinterpret_cast<const char*>(buff), 4);
}

uint32_t SSTD_LOCAL readUInt32(ifstream & in) {
	unsigned char buff[4];

	in.read(reinterpret_cast<char*>(buff), 4);

	uint32_t multiplier = 1, ret = 0;
	for (int i = 0; i < 4; i++) {
		ret += multiplier * buff[i];
		multiplier *= 0x100;
	}
	return ret;
}

void SSTD_API raw_save(const char * filename, const void * data, unsigned int bitDepth, unsigned int width, unsigned int height ) {
	using namespace std;

	const string fn = fix_filename(filename, ".raw");
	ofstream fout(fn, ios::out | ios::binary | ios::trunc);
	if (!fout) {
		throw runtime_error(std::string("Could not open file!") + fn);
	}

	/* Saving in RAW file format */
	fout.write("RAW", 3); // Magic number	
	writeUInt32(fout,width);
 	writeUInt32(fout,height);
	writeUInt32(fout,bitDepth/8);

	fout.write( reinterpret_cast<const char*>(data), (width*height+1)*(bitDepth/8));

	fout.close();
}


void SSTD_API raw_load(const char * filename, void * data, unsigned int &bitDepth, unsigned int &width, unsigned int &height ) {
	using namespace std;

	const string fn = fix_filename(filename, ".raw");
	ifstream fin(fn, ios::in | ios::binary );
	if (!fin) {
		throw runtime_error(std::string("Could not open file: ") + fn);
	}

	/* Loading in RAW file format */
	char buf[4] = {};
	fin.read(buf, 3);
	if( strcmp(buf, "RAW")) {
		throw runtime_error(std::string("Not RAW file: ") + fn);
	}

	uint32_t w,h,d;
	w = readUInt32(fin);
 	h = readUInt32(fin);
	d = readUInt32(fin);
	if (data) {
		fin.read( reinterpret_cast<char*>(data), (width*height+1)*(d));
		if (!fin)
			cout << "[Error}" << endl;
	}
	width = w;
	height = h;
	bitDepth = d*8;

	fin.close();
}
