#include "stat.h"

#include <iostream>
#include <iomanip>


std::string Benchmark::printGraph(unsigned int val) {
	std::string str = " ";
	for ( unsigned long long i = 0; i < val*100/total_obs; i++) {
		str.push_back('*');
	} 
	return str;
}

void Benchmark::addObservation(unsigned int val) {
	if (val < scale_min) { 
		freq[0]++;
	} 
	else if (val >= scale_max) {
		freq[size+1]++;
	}
	else {
		freq[(val-scale_min)/scale_step + 1]++;
	}

	if (val < min) min = val;
	else if (val > max) max = val;

	total_obs++;
	total += val;
}


Benchmark::Benchmark( unsigned int scale_step,	unsigned int scale_min,	unsigned int scale_max) :
	scale_step(scale_step),
	scale_min(scale_min),
	scale_max(scale_max),
	size((scale_max - scale_min)/scale_step),
	min(-1),
	max(0),
	total_obs(0),
	total(0),
	sq_diff(0)
{
freq = new unsigned int[ size + 2 ](); 
}

Benchmark::~Benchmark() {
	delete[] freq;
}



void Benchmark::printObservations() {
	using namespace std;
	const int fcol = 10;
	cout << setw(fcol) << ("<" + to_string(scale_min)) << ":" << setw(10) << freq[0] << printGraph(freq[0]) << '\n';
	for ( int i = 0; i < size; i++) {
			cout << setw(fcol) << (to_string(scale_min+i*scale_step) + "-" + to_string(scale_min+(i+1)*scale_step -1)) << ":" << setw(10) << freq[i+1] << printGraph(freq[i+1]) << '\n';
	}
	cout << setw(fcol) << (">=" + to_string(scale_max)) << ":" << setw(10) << freq[size+1] << printGraph(freq[size+1]) << '\n';
	

	cout << "----------------------------------------------" << endl;
	cout << "Min:\t"<<min << "\tMax:\t" << max << endl;
	cout << setprecision(5);
	cout << "Total Observations: " << total_obs << "\tAverage: " << total/static_cast<double>(total_obs) << "\tsd: " << sqrt(sq_diff/total_obs) << endl;
}



