#include <iostream>
#include <algorithm>
#include <cstdlib>
#include <ctime>
#include "lvq.h"
#include "progress_callback.h"
#include "SOM.h"
#include <iomanip>
#include <thread>
#include <atomic>
#include <unistd.h>

#include <fstream>

using namespace std;

int print_statistics(const Codebook & cb_in) {
	
		cout << "Labels: \n";

		vector<pair<string, int>> labels(cb_in.getLabelMap().size());

		int maxlen = 0;
		for (auto label : cb_in.getLabelMap()) {
			labels[label.second] = make_pair( label.first, 0);
			if (label.first.size() > maxlen) {
				maxlen = label.first.size();
			}
		}

		for ( auto cw : cb_in ) {
			labels[cw.label()].second++;			
		}
			
		for ( auto entry : labels ) {
			cout << setw(maxlen + 4) << entry.first << " :  " << entry.second << '\n';
		}
		cout << flush;
	
	return 0;
}


void test_1( const VQ & encoder, const Codebook & training_data, const std::vector<unsigned int> &label_map, const int begin, const int end, atomic<int> & hits, atomic<int> & counter ) {
	for (int i = begin; i < end; i++ ) { 
		auto & elem = training_data[i];
		if (label_map[elem.label()] == encoder.encode(elem) )
			hits++;
		counter++;
	}
}

int test(const VQ & encoder, const Codebook & training_data) {
	std::vector<thread> threads;
	atomic<int> hits(0), counter(0);
	
	int nthreads = 1;//std::thread::hardware_concurrency();
	int part 	 = training_data.size()/nthreads;
	
	std::vector<unsigned int> label_map(training_data.getLabelMap().size());
	for ( unsigned int i = 0; i < label_map.size(); i++) {
		label_map[i] = encoder.getLabelId( training_data.getLabelString(i) );
	}
	
	for ( int i = 0; i < nthreads-1; i++) {
		threads.emplace_back( thread(test_1, std::ref(encoder), std::ref(training_data), std::ref(label_map), i*part, (i+1)*part, std::ref(hits), std::ref(counter)));
	}
	threads.emplace_back( thread(test_1, std::ref(encoder), std::ref(training_data), std::ref(label_map), (nthreads-1)*part, training_data.size(), std::ref(hits), std::ref(counter)));
	
	ProgressCallback p_cb;
	while (counter < training_data.size()) {
		p_cb.updateProgress(static_cast<double>(counter*100)/training_data.size(), " Testing: ");
		usleep(1000);
	}
	p_cb.updateProgress(100, " Testing: ");
	
	
	for ( auto & th : threads ) {
		th.join();
	}
	return hits;
}


int main() {

	srand(time(0));
	ofstream fout;
	
	//LOW_DIMENSION SOM TEST	
	// Generate a number of clusters.
	const int N_CLUSTERS = 10;
	const int SPREAD = 1000;
	const int DIMENSIONS = 2;
	
	std::vector<Vector> cluster_centres;
	
	cout << "Clusters: ";
	for ( int i = 0; i < N_CLUSTERS; i++) {
		cluster_centres.emplace_back( DIMENSIONS ); 
		
		for ( int j = 0; j < DIMENSIONS; j++) {
			cluster_centres.back()[j] = -SPREAD + rand() % (2*SPREAD);
		}
		
		cout << cluster_centres.back() << ", ";
	}
	cout << endl;
	
	cout << "Clusters2: ";
	for ( int i = 0; i < N_CLUSTERS; i++) {
			cout << cluster_centres[i] << ", ";
	}
	cout << endl;
	
	
	// Populate training data around clusters.
	Codebook training_data;
	
	unsigned int SAMPLES = 10*N_CLUSTERS;
	
	cout << "Sample: ";
	for ( int i = 0; i < SAMPLES; i++) {
		int cluster = rand() % N_CLUSTERS;
		Vector v = cluster_centres[cluster];
		
		for ( int j = 0; j < DIMENSIONS; j++) {
			v[j] += -SPREAD/10 + rand() % (2*SPREAD/10);
		}

		training_data.addCodeword( v, to_string(cluster));
		
		cout << training_data[training_data.size()-1] << ", ";
	}
	cout << endl;
	
	fout.open("training_data.csv");
	for ( auto v : training_data ) {
		fout << to_string(v) << endl;
	}
	fout.close();
	
	// CREATE SOM:
	ProgressCallback pcb;	
	Codebook result;
	SOM map(N_CLUSTERS/2, 3, &pcb);
	
	map.train(training_data, 100*training_data.size());
	
	// Save som state for debugging.
	fout.open("som.csv");
	for ( auto v : map ) {
		fout << to_string(v) << endl;
	}
	fout.close();
	
	LVQ lvq(map.tag_and_purge(training_data));
	
	fout.open("stripped_som.csv");
	for ( auto v : lvq ) {
		fout << to_string(v) << endl;
	}
	fout.close();
	
	cout << lvq << endl;
	
	print_statistics(lvq);
	
	int hits = test(lvq, training_data);
	cout << "SOM: HitRatio: " << static_cast<double>(hits)/training_data.size() << endl;
	
	lvq.train_vq(training_data, training_data.size()*100);
	
	fout.open("lvq.csv");
	for ( auto v : lvq ) {
		fout << to_string(v) << endl;
	}
	fout.close();
	
	hits = test(lvq, training_data);
	cout << "LVQ: HitRatio: " << static_cast<double>(hits)/training_data.size() << endl;
	
	
}




