from heapq import *

#Global Params
INERT = '-'


def align(x, y, pos):
    if pos < 0:
        y = INERT*abs(pos) + y
        x = x[:]
    else:
        x = INERT*pos + x
        y = y[:]
    return x,y


def _combine(x, y, pos):
    r = ''
    x,y = align(x,y,pos)
    
    i = 0
    stop = min(len(x), len(y))
    while i < stop:
        if x[i] == y[i]:
            r += x[i]
        elif x[i] == INERT:
            r += y[i]
        elif y[i] == INERT:
            r += x[i]
        else:
            r += INERT
        i+=1

    if len(x) < len(y):
        longest = y
    else:
        longest = x

    r += longest[i:]
    return r

def _not_inert(x):
        return x != INERT

def non_inert(x):
    return sum(map(_not_inert, x))

def hamming_dist(x, y):
    score = 0
    for i in range(min(len(y), len(x))):
        if x[i] == y[i] and not (x[i] == INERT):
            score += 1
    return score

def slide_dist(_x,_y, d):
    x = _x[:]
    y = _y[:]
    max_score = d(x,y)
    pos = 0
    for i in range(len(x)-1):
        y = INERT + y
        score = d(x,y)
        if score > max_score:
            max_score = score
            pos = -i -1

    y = _y
    
    for i in range(len(y)-1):
        x = INERT + x
        score = d(x,y)
        if score > max_score:
            max_score = score
            pos = i +1
    
            
    return max_score, pos

def max_hamming_dist(x, y):
    return slide_dist(x,y,hamming_dist)

def dist(x,y):
    return max_hamming_dist(x,y)

def combine( _samples, sequence_length, INERT='-'):
    # Algorithm outline
    # 1. Find the two elements a,b in samples maximizing dist.
    # 2. Remove them from the list. And add combine(a,b)
    # 3. Repeat from 1 til all

    samples = _samples[:]
    N_SAMPLES = len(samples)

    a = None
    b = None
    pos = None
    distance = 0

    used = set()
    best = set()

    heap = []

    # Build priority queue
    for i in range(N_SAMPLES):
        for j in range(i+1, N_SAMPLES):
            d, p = dist(samples[i], samples[j])
            heappush(heap, (-d, p, i, j))

    # Each step reduces the searchspace by 1.
    for i in range(N_SAMPLES-1):
        d, p, a, b = heappop(heap)
        while ( a in used or b in used):
            d, p, a, b = heappop(heap)

        # Consume elements
        used.add(a)
        used.add(b)

        # Add new element
        samples.append(_combine( samples[a], samples[b], p))

        # Populate heap
        for i in range(len(samples)-1):
            if (i not in used ):
                d, p = dist(samples[i], samples[-1])
                heappush(heap, (-d, p, i, len(samples)-1))

        n = non_inert(samples[-1])
        if (n > sequence_length//2):
            best.add((n,samples[-1]))

    return best_guess(best)

# From candidates generate best guess
def best_guess(best):
    guesses = list(best)
    guesses.sort()
    guesses = [ v for n,v in guesses ]

    # Pick the top one and align all others to it.
    primus = guesses.pop()

    for i in range(len(guesses)-1, -1, -1):
        d,p = dist(primus, guesses[i])
        primus, guesses[i] = align(primus, guesses[i], p)

    for i in range(len(guesses)-1, -1, -1):
        d,p = dist(primus, guesses[i])
        primus, guesses[i] = align(primus, guesses[i], p)

    guesses.append(primus)

    guess = [ [] for i in range(max(map(len, guesses))) ]
    alphabet = { INERT : -1}
    char = 0

    for x in guesses:
        print(x)

    # Vote final string
    for x in guesses:
        for i in range(len(x)):
            p = alphabet.setdefault(x[i], char)
            if p == char: char += 1

            if p != -1:
                while (len(guess[i]) < p+1):
                    guess[i].append(0);
                guess[i][p] += 1

    rev_alpha = dict(zip(alphabet.values(), alphabet.keys()))

    # Tally votes.
    ret = ''
    for x in guess:
        _max = -1
        for i in range(0, len(x)):
            if _max == -1 or x[i] > x[_max]:
                _max = i

        ret += rev_alpha[_max]

    print(guess)

    print()
    print(ret)

    return ret
