#include "simple_rsa.h"



void rsa_encrypt( BIGNUM *ciphertext, const BIGNUM *plaintext, const BIGNUM *n ) {
	BIGNUM *multiplier;
	BN_CTX *ctx;	
	
	multiplier = BN_dup( plaintext );
	
	if (ciphertext != plaintext) 
		BN_copy(ciphertext, plaintext);

	ctx = BN_CTX_new();
	BN_CTX_init(ctx);

	BN_mod_sqr(multiplier, multiplier, n, ctx);
   	BN_mod_mul(ciphertext, ciphertext, multiplier, n, ctx);

    BN_clear_free(multiplier);
    BN_CTX_free(ctx);
    
}

void rsa_decrypt( BIGNUM *plaintext, const BIGNUM *ciphertext, const BIGNUM *n, const BIGNUM *d ) {
	mod_exp( plaintext, ciphertext, d, n );
}

void mod_exp( BIGNUM *ret, const BIGNUM *base, const BIGNUM *exponent, const BIGNUM *mod ) {
    int nbits, i;
	BIGNUM *multiplier;
	BN_CTX *ctx;	
	
	multiplier = BN_dup( base );
	BN_one(ret); 	// OK, even if base == ret, as we made a copy above.

	ctx = BN_CTX_new();
	BN_CTX_init(ctx);

	i = 0;    
    nbits = BN_num_bits(exponent);
    while ( i < nbits ) {
    	if ( BN_is_bit_set(exponent, i) ) {
    		BN_mod_mul(ret, ret, multiplier, mod, ctx);
    	}
    	BN_mod_sqr(multiplier, multiplier, mod, ctx);
    	++i;
    }
    
    BN_clear_free(multiplier);
    BN_CTX_free(ctx);
}

