//#include "../crypto/options.h"
#include <stdio.h>
#include <time.h>
#include <openssl/rand.h>
#include <openssl/bn.h>
#include <openssl/rsa.h>
#include <signal.h>
#include <string.h>



void loop_init( BIGNUM *ret, const BIGNUM *base, const BIGNUM *exponent, const BIGNUM *mod );
void loop_mod_mul( BIGNUM *ret, const BIGNUM *base, const BIGNUM *exponent, const BIGNUM *mod );
void loop_mod_sqr( BIGNUM *ret, const BIGNUM *base, const BIGNUM *exponent, const BIGNUM *mod );

int exit_condition = 0;

void quit_cb(int signal) {
	exit_condition = 1;
}

#define INIT 0
#define MODMUL 1
#define MODSQR 2

int main(int argc, char ** argv) {
	int choice = -1;
	if (argc != 2) {
		printf("[Error] No training target supplied.\n");
		return -1;
	}


	if (!strcmp(argv[1],"init")) {
		choice = INIT;
	}
	else if	(!strcmp(argv[1],"mod_mul")) {
			choice = MODMUL;
	}
        else if (!strcmp(argv[1],"mod_sqr")) {
		choice = MODSQR;
	}
	else {
		printf("[Error] %s not a valid choice, must be \"init\", \"mod_mul\" or \"mod_sqr\".", argv[1]);
		return -1;
	}

	BIGNUM *base;
	int err;
	
	// Set signal handler.
	signal(SIGTERM, quit_cb);
	signal(SIGINT, quit_cb);
	signal(SIGABRT, quit_cb);

	// Seed random
	time_t t = time(0);
	RAND_seed(&t, sizeof(time_t));

	// Set encryption exponent.
	base = BN_new();
	BN_init(base);
	BN_set_word(base, 3);

	// Generate RSA keys
	RSA* rsa = RSA_new();
	RSA_generate_key_ex(rsa, 1024, base, NULL); 

	// Set initial ciphertext.
	BN_rand(base, 1024, 1, 0);


	printf("%s\n", "Starting loop"); 
	// Main loop.
	
	switch(choice) {
		case INIT:
			loop_init(base, base, rsa->n, rsa->d);
			break;
		case MODMUL:
			loop_mod_mul(base, base, rsa->n, rsa->d);
			break;
		case MODSQR:
			loop_mod_sqr(base, base, rsa->n, rsa->d);
			break;
	}

	BN_free(base);
	RSA_free(rsa);
}


