#include <stdio.h>
#include <time.h>
#include <openssl/rand.h>
#include <openssl/bn.h>
#include <openssl/rsa.h>
#include <string.h>
#include <fstream>
#include <iostream>
#include <unistd.h>

#include "spy.h"

const unsigned int SAMPLES = 1000000;

#define MEASURE( what, label ) \
	total_time = 0;\
    min_time = 0xffffffff;\
	for( int i = 0; i < SAMPLES; i++) {\
		raw_t t0 = getCCNT();\
		what;\
		raw_t dt = delta_time(t0, getCCNT()); \
		if (dt < min_time) min_time = dt; \
		total_time += dt; \
	}\
	fout << (label) << "_min:\t" << min_time << '\n'\
	     << (label) << "_avg:\t" << total_time/SAMPLES << '\n';

using namespace std;

int main(int argc, char ** argv) {

	ofstream fout("parameters");
	if (!fout) {
		cout << "[Error] Could not open file \"parameters\"" << endl;
		return 1;
	}
	
	BIGNUM *base;
	int err;

	// Seed random
	time_t t = time(0);
	RAND_seed(&t, sizeof(time_t));

	// Set encryption exponent.
	base = BN_new();
	BN_init(base);
	BN_set_word(base, 3);

	// Generate RSA keys
	RSA* rsa = RSA_new();
	RSA_generate_key_ex(rsa, 1024, base, NULL); 

	// Set initial ciphertext.
	BN_rand(base, 1024, 1, 0);

	BN_CTX *ctx = BN_CTX_new();
	BN_CTX_init(ctx);

	unsigned long long total_time = 0;
	raw_t min_time = 0;
	// Main loop
	
	MEASURE( usleep(0), "OS-TICKS");
	MEASURE( BN_mod_mul(base, base, rsa->d, rsa->n, ctx), "MOD_MUL");
	MEASURE( BN_mod_sqr(base, base, rsa->n, ctx), "MOD_SQR");
	
	BN_free(base);
	RSA_free(rsa);
	BN_CTX_free(ctx);
	fout.close();
}


