#include <openssl/bn.h>
#include "config.h"

extern int exit_condition;

void loop_init( BIGNUM *ret, const BIGNUM *base, const BIGNUM *exponent, const BIGNUM *mod ) {

	// Inform that we are ready.
	FILE* filp = fopen("begin_loop", "w");
	fprintf(filp, "%d", 1);
	fclose(filp);

	while (!exit_condition) {
		int nbits, i;
		BIGNUM *multiplier;
		BN_CTX *ctx;	

		multiplier = BN_dup( base );
		BN_one(ret); 	// OK, even if base == ret, as we made a copy above.

		ctx = BN_CTX_new();
		BN_CTX_init(ctx);

		i = 0;    
		nbits = BN_num_bits(exponent);
		BN_clear_free(multiplier);
		BN_CTX_free(ctx);
	}
}

void loop_mod_mul( BIGNUM *ret, const BIGNUM *base, const BIGNUM *exponent, const BIGNUM *mod ) {
	int nbits, i;
	BIGNUM *multiplier;
	BN_CTX *ctx;	

	multiplier = BN_dup( base );
	BN_one(ret); 	// OK, even if base == ret, as we made a copy above.

	ctx = BN_CTX_new();
	BN_CTX_init(ctx);

	i = 0;    
	nbits = BN_num_bits(exponent);

	// Inform that we are ready.
	FILE* filp = fopen("begin_loop", "w");
	fprintf(filp, "%d", 1);
	fclose(filp);
	
	while (!exit_condition) {
		BN_mod_mul(ret, ret, multiplier, mod, ctx);
	}

	BN_clear_free(multiplier);
	BN_CTX_free(ctx);	
}

void loop_mod_sqr( BIGNUM *ret, const BIGNUM *base, const BIGNUM *exponent, const BIGNUM *mod ) {
	int nbits, i;
	BIGNUM *multiplier;
	BN_CTX *ctx;	

	multiplier = BN_dup( base );
	BN_one(ret); 	// OK, even if base == ret, as we made a copy above.

	ctx = BN_CTX_new();
	BN_CTX_init(ctx);

	i = 0;    
	nbits = BN_num_bits(exponent);
	
	// Inform that we are ready.
	FILE* filp = fopen("begin_loop", "w");
	fprintf(filp, "%d", 1);
	fclose(filp);

	while (!exit_condition) {
		BN_mod_sqr(multiplier, multiplier, mod, ctx);
	}

	BN_clear_free(multiplier);
	BN_CTX_free(ctx);	
}
