#include <iostream>
#include "train_loop.h"
#include "raw.h"
#include "raw2diff.h"
#include "pgm.h"
#include <cstring>
#include <cmath>

using namespace std;

void transmit1();
void transmit0();



#define DEFAULT_FILENAME "default"
#define DEFAULT_SAMPLES 5

#define OPTION_SAMPLES "samples="

#define __STRINGIFY(x) # x
#define STRINGIFY(x) __STRINGIFY(x)

int process_arguments(int argc, char ** argv );

void print_help() {
	cout << "USAGE:  training [-o fname] [--samples=N] msg" << endl <<
"    fname   :   Output filename, this will create the files \"fname.raw\" and \"fname.pgm\". \n"
"                If no filename is given the default is \"" DEFAULT_FILENAME "\".\n"
"    N,R     :   N is the number of total samples. \n"
"                The resulting dataset will have dimensions N X CACHE_SETS.\n"
"                The defaults are N=" STRINGIFY(DEFAULT_SAMPLES) ".\n"
<< endl;
}


int samples = DEFAULT_SAMPLES;
const char * outfilename = DEFAULT_FILENAME;
const char * msg = nullptr;

int main(int argc, char ** argv ) {

	if ( process_arguments(argc, argv )) {
		return 1;
	}

	
	raw_t 	* data 	= new raw_t[ samples*(CACHE_SETS)]();
	
	cout 	<< "Stats: \n" 
			<< "   Diff size: " << DIFF_T_SIZE << endl
			<< "  Oufilename: " << outfilename << endl
			<< "           N: " << samples << endl
			<< "  Cache sets: " << CACHE_SETS << endl
			<< "     Message: " << msg << endl;
	
	run_loop(msg, samples, data );
	
	//raw_save(outfilename, data, 8*RAW_T_SIZE, CACHE_SETS, samples );
	diff_t *ddata = new diff_t[samples*CACHE_SETS];

	raw2diff(ddata, data, samples, CACHE_SETS);

	pgm_save(outfilename, ddata, DIFF_T_SIZE*8, CACHE_SETS, samples );

	delete [] data;
	delete [] ddata;
}

int i1, i0;

void transmit1(void) {
	i1 = i1 + cos(i1);
}

void transmit0(void){ 
	
}

int process_arguments(int argc, char ** argv ) {
	if (argc < 2) {
		print_help();
		return 1;
	}
	else {
		for (int i = 1; i < argc; i++ ) {
			const char * arg = argv[i];
			if ( *arg == '-' ) {
				if ( *(++arg) == '-') {
					++arg; 
					if (!strncmp(OPTION_SAMPLES, arg, strlen(OPTION_SAMPLES))) {
						arg += strlen(OPTION_SAMPLES);
						samples = atoi(arg);
					}
					else {
						cout << "Unrecognized option: " << argv[i] << endl << endl;
						print_help();
						return 1;
					}
				}
				else if (*arg == 'o') { // -o
					if ( *(++arg) != 0) {
						outfilename = arg;
					}
					else if ( *(arg) == '\0' && ++i < argc ) {
						outfilename = argv[i];
					}
					else {
						cout << "No filename supplied to -o. " << endl;
						print_help();
						return 1;
					}
				}
				else {
					cout << "Unrecognized option: " << argv[i] << endl << endl;
					print_help();
					return 1;
				}
			}
			else {
				if (msg == nullptr) {
					msg = arg;
				} else {
					cout << "Multiple messages given, must be exactly 1. \nCurrent argument: " << argv[i] << endl << "Previous message argument: " << msg << endl;
					print_help();
					return 1;
				}
			}
		}
		
		if (msg == nullptr ) {
			cout << "No message given." << endl << endl;
			print_help();
			return 1;
		}
		return 0;
	}
}












