#include "simple_rsa.h"
#include <cstring>
#include <cstdio>
#include <ctime>
#include <openssl/rand.h>
#include <openssl/rsa.h>
#include <openssl/bn.h>
#include "raw2diff.h"
#include "asm_defines.h"
#include "pgm.h"

#define SIZE 1024
#define BN_INIT(bn) bn = BN_new(); BN_init(bn)

const char * USAGE = "Usage: rsa-sync keys outfilename\n";

int main(int argc, char ** argv) {
	char  key_p[SIZE], key_q[SIZE],  key_n[SIZE],  key_e[SIZE],  key_d[SIZE];
	BIGNUM *msg = NULL;
	raw_t *raw_data = NULL;
	diff_t*dif_data = NULL;

	time_t t = time(0);
	RAND_seed(&t, sizeof(time_t));

	if (argc != 3) {
		printf("Wrong number of arguments, should be 2.\n");
		printf(USAGE);
		return 1;
	}

	// Open files.
	FILE* filp = fopen(argv[1], "r");
	if (!filp) {
		printf("Error, could not open keyfile: %s\n", argv[1]);
		return 1;
	}
	
	if (!fgets(key_p, SIZE, filp)) { printf("Keyfile: Bad format.\n"); return 1; }
	if (!fgets(key_q, SIZE, filp)) { printf("Keyfile: Bad format.\n"); return 1; }
	if (!fgets(key_n, SIZE, filp)) { printf("Keyfile: Bad format.\n"); return 1; }
	if (!fgets(key_e, SIZE, filp)) { printf("Keyfile: Bad format.\n"); return 1; }
	if (!fgets(key_d, SIZE, filp)) { printf("Keyfile: Bad format.\n"); return 1; }

	// Prepare Key.
	RSA *rsa = RSA_new();

	BN_INIT(rsa->p);
	BN_INIT(rsa->q);
	BN_INIT(rsa->n);
	BN_INIT(rsa->e);
	BN_INIT(rsa->d);

	if (!BN_hex2bn(&rsa->p, key_p)) { printf("Keyfile: Bad format.\n"); goto HEX_LOAD_FAIL; }
	if (!BN_hex2bn(&rsa->q, key_q)) { printf("Keyfile: Bad format.\n"); goto HEX_LOAD_FAIL; }
	if (!BN_hex2bn(&rsa->n, key_n)) { printf("Keyfile: Bad format.\n"); goto HEX_LOAD_FAIL; }
	if (!BN_hex2bn(&rsa->e, key_e)) { printf("Keyfile: Bad format.\n"); goto HEX_LOAD_FAIL; }
	if (!BN_hex2bn(&rsa->d, key_d)) { printf("Keyfile: Bad format.\n"); goto HEX_LOAD_FAIL; }

	if (RSA_check_key(rsa) != 1)  { printf("Keyfile: Bad format.\n"); goto HEX_LOAD_FAIL; }

	fclose(filp); // No longer needed.

	// Generate random message

	msg = BN_new();
	BN_init(msg);
	BN_rand(msg, 1024, 1, 0);

	// Prepare Output buffers.
	raw_data = (raw_t*) malloc(SIZE*RAW_T_SIZE*CACHE_SETS);
	dif_data = (diff_t*)malloc(SIZE*DIFF_T_SIZE*CACHE_SETS);

	// Touch buffer.
	memset(raw_data, 0, SIZE*RAW_T_SIZE*CACHE_SETS);

	// Run spy.
	rsa_decrypt( msg, msg, rsa->n, rsa->d, raw_data );
	
	// Save result
	raw2diff(dif_data, raw_data, SIZE, CACHE_SETS);

	pgm_save(argv[2], dif_data, DIFF_T_SIZE*8, CACHE_SETS, SIZE);

	// Cleanup
	free(raw_data);
	free(dif_data);
	BN_free(msg);
	RSA_free(rsa);

	return 0;
HEX_LOAD_FAIL:
	RSA_free(rsa);
	return 1;
}

