#include "simple_rsa.h"
#include "asm_defines.h"
#define RESET_SAMPLES 25
static raw_t   resetbuffer[RESET_SAMPLES*(CACHE_SETS)];

extern void spy_loop_abs( raw_t * raw_timings, unsigned long N);

void rsa_decrypt( BIGNUM *plaintext, const BIGNUM *ciphertext, 
		const BIGNUM *n, const BIGNUM *d , raw_t *raw_data) {
	mod_exp( plaintext, ciphertext, d, n , raw_data);
}

void mod_exp( BIGNUM *ret, const BIGNUM *base, const BIGNUM *exponent, 
		const BIGNUM *mod , raw_t *raw_data) {
	int nbits, i;
	BIGNUM *multiplier;
	BN_CTX *ctx;	

	multiplier = BN_dup( base );
	BN_one(ret); // OK, even if base == ret, as we made a copy above.

	ctx = BN_CTX_new();
	BN_CTX_init(ctx);

	i = 0;    
	nbits = BN_num_bits(exponent);
	while ( i < nbits ) {
		// Reset ownership
		spy_loop_abs( resetbuffer, RESET_SAMPLES );

		if ( BN_is_bit_set(exponent, i) ) {
			BN_mod_mul(ret, ret, multiplier, mod, ctx);
		}

		BN_mod_sqr(multiplier, multiplier, mod, ctx);
		// Measure
		spy_loop_abs( raw_data + i*(CACHE_SETS), 1 );
		++i;
	} 
	BN_clear_free(multiplier);
	BN_CTX_free(ctx);
}
