#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/workqueue.h>
#include <linux/slab.h>
#include <linux/cpumask.h>

#define _stringify(x) # x
#define stringify(x) _stringify(x)

void schedule_on_all_processors( const char * wq_name, void (*workerfn)(struct work_struct *)) {
	struct workqueue_struct * wq;
	struct work_struct * work;
	int i;
	
	work = (struct work_struct*) kmalloc( sizeof(struct work_struct)*nr_cpu_ids, GFP_KERNEL);
	wq = create_workqueue( wq_name );
	
	i = 0;
	while (i < nr_cpu_ids) {
		if (cpu_online(i)) {
			INIT_WORK(work + i, workerfn); 
			queue_work_on(i, wq, work + i);
		}
		else {
			printk(KERN_INFO "Skipping CPU%d: Offline.", i);
		}
		
		i++;
	}
	
	flush_workqueue( wq );
	destroy_workqueue( wq );
	kfree(work);
}

static void enable_usermode(struct work_struct * work);
static void disable_usermode(struct work_struct * work);

static int mykmod_init(void)
{
	printk(KERN_INFO "Init!\n");
	schedule_on_all_processors("wq-eut-enable", enable_usermode);
	printk(KERN_INFO "Init Done!\n");
	return 0;
}


 
static void mykmod_exit(void)
{
	printk(KERN_INFO "Exit!\n");
	schedule_on_all_processors("wq-eut-reset", disable_usermode);
	printk(KERN_INFO "Exit Done!\n");
}
 
module_init(mykmod_init);
module_exit(mykmod_exit);

MODULE_DESCRIPTION(stringify(MODNAME));
MODULE_LICENSE("GPL");

static inline void init_perfcounters (int32_t do_reset, int32_t enable_divider);

static void enable_usermode(struct work_struct * work) {

	init_perfcounters (1, 0);

	 /* enable user-mode access to the performance counter*/
	asm  ("MCR p15, 0, %0, C9, C14, 0\n\t" :: "r"(1));

	/* disable counter overflow interrupts (just in case)*/
	asm ("MCR p15, 0, %0, C9, C14, 2\n\t" :: "r"(0x8000000f));

	printk(KERN_INFO "Enable, ran on  cpu: %d.\n", smp_processor_id() );
}

static void disable_usermode(struct work_struct * work) {
	
	
	
	/* reset user-mode access */
	asm  ("MCR p15, 0, %0, C9, C14, 0\n\t" :: "r"(0));

	/* reset counter overflow interrupts (just in case)*/
	asm ("MCR p15, 0, %0, C9, C14, 2\n\t" :: "r"(0x0));

	printk(KERN_INFO "Reset, ran on  cpu: %d.\n", smp_processor_id() );
}

static inline void init_perfcounters (int32_t do_reset, int32_t enable_divider)
{
  // in general enable all counters (including cycle counter)
  int32_t value = 1;

  // peform reset:  
  if (do_reset)
  {
    value |= 2;     // reset all counters to zero.
    value |= 4;     // reset cycle counter to zero.
  } 

  if (enable_divider)
    value |= 8;     // enable "by 64" divider for CCNT.

  value |= 16;

  // program the performance-counter control-register:
  asm volatile ("MCR p15, 0, %0, c9, c12, 0\t\n" :: "r"(value));  

  // enable all counters:  
  asm volatile ("MCR p15, 0, %0, c9, c12, 1\t\n" :: "r"(0x8000000f));  

  // clear overflows:
  asm volatile ("MCR p15, 0, %0, c9, c12, 3\t\n" :: "r"(0x8000000f));
}



