#include <iostream>
#include <cstring>
#include <iomanip>
#include <unistd.h>
#include <csignal>

#include "spy.h"
#include "stat.h"
#include "raw.h"
#include "pgm.h"


using namespace std;


const int DEFAULT_N = 1000;
const int ROUNDS = 10;


void save( raw_t * timing, int n, const char * fn_base ) noexcept;

void setParam( int & out, const char * arg, unsigned int min, unsigned int max , const char * tag) {
	int temp = atoi(arg);
	if (temp >= min && temp <= max )
		out = temp;
	else {
		cerr << "Argument ["<< tag << "] malformed: Not integer between "<< min <<" and " << max << endl;
		exit(1);
	}
}

void printHelp() {
	cout 	<< "USAGE: spy fname [N].\n"
		<< "\tfname- base filename of output files.\n"
		<< "\tN    - number of consequtive samples in one run.\n"
		<< endl;
}

const unsigned long CLOCK_TYPE_MAX = static_cast<unsigned long>((static_cast<unsigned long long>(0x1) << (RAW_T_SIZE*8))-1);


volatile sig_atomic_t exit_condition = 0;

void program_interupted(int sig) {
	exit_condition = 1;
}

int main(int argc, const char ** argv) {
	int n = DEFAULT_N;

	signal( SIGINT, program_interupted);

	if (argc < 2 || (argc > 1 && !strcmp("--help", argv[1] ))) { 
		printHelp(); return 0; 
	}
	const char * fn = argv[1];

	if (argc > 2) setParam(n	, argv[2],	1, 		0x7fffffff	, "N");	

	const int C0 = 12, C1 = 8;
	cout 	<< "Running "<< argv[0] <<" with parameters:\n"
		<< setw(C0) << "Cache size:" 	<< setw(C1) << CACHE_SIZE 
		<< setw(C0) << "Line size:" 	<< setw(C1) << CACHE_LINE 	<< endl 
		<< setw(C0) << "Cache sets:" 	<< setw(C1) << CACHE_SETS 
		<< setw(C0) << "Cache Ass:" 	<< setw(C1) << CACHE_ASSOC 	<< endl
		<< setw(C0) << "N:" 			<< setw(C1) << n  
		<< setw(C0) << "Fname:"			<< setw(C1) << fn  
		<< "\n--------------------------------------------------" << endl;



	raw_t * timing = new raw_t[CACHE_SETS*(n + ROUNDS)]();

	// Warmup
	spy_loop_abs(timing, ROUNDS);

	for (int i = 0; i < n; i++) {
		// Spy
		spy_loop_abs(timing + i*CACHE_SETS, ROUNDS);
		usleep(0);
	}

	// Save current run.
	save( timing, n, fn );

	delete [] timing;


	return 0;
}

void save( raw_t * timing, int n, const char * fn_base ) noexcept {
	try {
		cout << "Saving raw timings to " <<  fn_base << ".raw ... "<< flush;
		raw_save(fn_base, timing, RAW_T_SIZE*8, CACHE_SETS, n);
		cout << "Done! " << endl;
	} catch (std::exception & e) {
		cout << e.what() << endl;
	} catch (...) {
		cout << "Unhandled exception!!!!! AAAAAH!" << endl;
	}
}



