#include <iostream>
#include <string>
#include <getopt.h>
#include <cassert>
#include "raw2diff.h"
#include "types.h"
#include "raw.h"
#include "pgm.h"


using namespace std;

#define  DEFAULT_OUT "out.pgm"
const int OPT_OUT = 'o';

const char * USAGE = ""
"raw2pgm [-o outfilename] file...\n"
"  Convert one or more RAW files into a PGM file.\n"
" -o     : Create the resulting pgm image with the supplied filename\n"
"           instead of the default \"" DEFAULT_OUT "\".\n";



int main(int argc, char ** argv ) {
	const char * outname = DEFAULT_OUT;
	
	while ( OPT_OUT == getopt( argc, argv, "o:" )) {
		outname = optarg;
	}
	
	if ( optind != argc-1 ) {
		cout << "[Error] Wrong number of arguments." << USAGE << endl;
		return -1;
	} 
	
	unsigned int height = 0, width = 0, bits = 0;

	// Determine total lines.
	raw_load(argv[optind], nullptr, bits, width, height );
	
	raw_t  * raw_data  = new raw_t [width*height];
	diff_t * diff_data = new diff_t[width*height];

	// Load all data
	raw_load(argv[optind], raw_data, bits, width, height );
	
	raw2diff(diff_data, raw_data, height, width);
	
	pgm_save( outname, diff_data, sizeof(diff_t)*8, width, height);
	
	delete [] raw_data;
	delete [] diff_data;
	
	return 0;

}
