#include <cstring>
#include <iostream>
#include <cstdio>
#include <cassert>
#include <openssl/rsa.h>
#include <openssl/bn.h>
#include "pgm.h"
#include <vector>
#include <string>
#include "types.h"
#include "asm_defines.h"

#define BN_INIT(bn) bn = BN_new(); BN_init(bn)

const char * USAGE = ""
"Usage: rsa-split \n"
"    Loads all in keys subfolder and matches it with the measurements in the current folder.\n";

using namespace std;

std::vector<diff_t> ones, zeros;
int width = 0; // To assert all line widths.

#define SIZE 1024

/** returns 0 on error, or positive integer with number of measurements. */
int load(const std::string & key_file, const std::string &measurement_file) {
	char  key_p[SIZE], key_q[SIZE],  key_n[SIZE],  key_e[SIZE],  key_d[SIZE];
	diff_t*dif_data = NULL;

	// Open files.
	FILE* filp = fopen(key_file.c_str(), "r");
	if (!filp) {
		printf("Error, could not open keyfile: %s\n", key_file.c_str());
		return 0;
	}
	
	if (!fgets(key_p, SIZE, filp)) { printf("Keyfile: Bad format.\n"); return 0; }
	if (!fgets(key_q, SIZE, filp)) { printf("Keyfile: Bad format.\n"); return 0; }
	if (!fgets(key_n, SIZE, filp)) { printf("Keyfile: Bad format.\n"); return 0; }
	if (!fgets(key_e, SIZE, filp)) { printf("Keyfile: Bad format.\n"); return 0; }
	if (!fgets(key_d, SIZE, filp)) { printf("Keyfile: Bad format.\n"); return 0; }

	fclose(filp); // No longer needed.
	
	// Prepare Key.
	BIGNUM *d;
	BN_INIT(d);

	if (!BN_hex2bn(&d, key_d)) { 
		printf("Keyfile: Bad format.\n"); 
		BN_free(d);
		return 0;
	}

	// Read measurements 
	unsigned int w, h, b;
	try {
		pgm_load(measurement_file.c_str(), nullptr, b, w, h);
		if (width == 0) width = w;
		assert(w == width);
		dif_data = new diff_t[w*h];
		pgm_load(measurement_file.c_str(), dif_data, b, w, h);
	} catch(...) {
		BN_free(d);
		cout << "Could not process measurement file: "<< measurement_file << endl;
		delete[] dif_data;
		return 0;
	}

	unsigned int pos = 0;
    unsigned int nbits = BN_num_bits(d);

	cout << "h: " <<  h <<" nbits: "<< nbits <<endl;
//	assert(nbits == h );

    for (unsigned int i = 0; i < nbits; i++ ) {
    	if ( BN_is_bit_set(d, i) ) {
    		for (int c = 0; c < w; c++) {
				ones.push_back(dif_data[pos++]);
			}
		}
		else {
    		for (int c = 0; c < w; c++) {
				zeros.push_back(dif_data[pos++]);
			}
		}
	}


	// Cleanup
	delete [] dif_data;
	BN_free(d);

	return h;
}

int main() {
	const char *key_pattern = "keys/key%02d";
	const char *measurement_pattern = "case%02d.pgm";
	char key_fn[256], measurement_fn[256];

	int i = 0;
	do {
		snprintf( key_fn, 256, key_pattern, i);
		snprintf( measurement_fn, 256, measurement_pattern, i);
		i++;
	} while (load( key_fn, measurement_fn));

	pgm_save("ones.pgm", ones.data(), DIFF_T_SIZE*8, width, ones.size()/width);
	pgm_save("zeros.pgm", zeros.data(), DIFF_T_SIZE*8, width, zeros.size()/width);

}

