#include "loader.h"
#include "raw.h"
#include "codebook_reduce.h"
#include "pgm.h"
#include "types.h"
#include "raw2diff.h"

#include <utility>

using namespace std;

Codebook & Loader::getCodebook(const std::string & tag) {
	auto it = codebooks.find(tag);
	if ( it == codebooks.end() ) {
	   it = codebooks.insert(make_pair<std::string, Codebook*>(string(tag), new Codebook())).first;
	}
	
	return *(it->second);
}

int Loader::loadRaw( const std::string & fn, const std::string & tag) {
    unsigned int bitdepth, width, height;

	// Poll sizes.
	raw_load(fn.c_str(), nullptr, bitdepth, width, height );

	raw_t * absData = new raw_t [width*height];
	diff_t* diffData= new diff_t[width*height];

	raw_load(fn.c_str(), absData, bitdepth, width, height );
    raw2diff(diffData, absData, height, width );

	Codebook & cb = getCodebook(tag);
    Vector temp(width);		
	for ( int r = 0; r < height; r++) {
		for ( int c = 0; c < width; c++) {
			temp[c] = diffData[r*width + c];
		}
		
		cb.addCodeword(temp, tag);
	}
	
	delete [] absData;
	delete [] diffData;

	return height;	
}

int Loader::loadPgm( const std::string & fn, const std::string & tag) {
    unsigned int bitdepth, width, height;

	// Poll sizes.
	pgm_load(fn.c_str(), nullptr, bitdepth, width, height );

	diff_t* diffData= new diff_t[width*height];

	pgm_load(fn.c_str(), diffData, bitdepth, width, height );

	Codebook & cb = getCodebook(tag);
    Vector temp(width);		
	for ( int r = 0; r < height; r++) {
		for ( int c = 0; c < width; c++) {
			temp[c] = diffData[r*width + c];
		}
		
		cb.addCodeword(temp, tag);
	}
	
	delete [] diffData;

	return height;	
}

int Loader::loadCodebook(const std::string & fn ) {
	Codebook temp = ::loadCodebook(fn);
	
	for (auto & cw : temp) {
	    std::string label = temp.getLabelString(cw.label());
	    getCodebook(label).addCodeword(cw, label);
	}
	
	return temp.size();
}


/** Reduce all codewords */
void Loader::reduce() {
    std::vector<const Codebook *> cbs;
    for (auto & elem : codebooks){
        cbs.push_back(elem.second);
    }
    
    if (reduced_components.empty() && !cbs.empty()) {
	   reduced_components = find_significant_components(cbs);
	}
    
    for (auto & cb_elem : codebooks ) {
        for (auto & cw : *cb_elem.second ) {
            cw = reduce_dimensions(cw, reduced_components);
        }
    }
}

Codebook Loader::combineAll() {
	Codebook out;
	for (auto & elem : codebooks){
        out.combine(*(elem.second));
    }
    return out;
}
